/**
  ******************************************************************************
  * @file         afe.c
  * @brief        This file provides code for analog front end.
  *               
  ******************************************************************************
  * @attention
  *
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include <zephyr.h>
#include <drivers/spi.h>
#include <devicetree.h>
#include <drivers/gpio.h>

#include "afe.h"

/* Private typedef -----------------------------------------------------------*/


/* Private define ------------------------------------------------------------*/
// set this to the number of rows below that we want to init.
#define NUM_AFE_REGISTERS_TO_INIT 37
// DELAY is defined as a register address that's not accesable as a trigger
#define DELAY 0x07

//testing?
#define MY_SPIM DT_NODELABEL(spi2)


/* Private macro -------------------------------------------------------------*/


/* Private function prototypes -----------------------------------------------*/
void ecg_afe_init(void);
void afe_print_all(void);
uint32_t ecg_read(int16_t array[]);

/* Private variables ---------------------------------------------------------*/

static const struct device * spi_dev;

 struct spi_cs_control spim_cs_one = { 
    .gpio_pin = DT_GPIO_PIN(MY_SPIM, cs_gpios),
    .gpio_dt_flags = GPIO_ACTIVE_LOW,
    .delay = 0,
  };

static struct spi_cs_control spi_cs = {
    .gpio_dev = NULL,
    .gpio_pin = 25,
    .gpio_dt_flags = GPIO_ACTIVE_LOW,
    .delay = 0,
  };

static struct spi_config spi_cfg = {
	.operation = SPI_WORD_SET(8) | SPI_TRANSFER_MSB,// |
		    // SPI_MODE_CPOL | SPI_MODE_CPHA,
	.frequency = 8000000,
	.slave = 0,
	.cs = &spi_cs,
};

const struct device *gpioPort0;


/* Private user code ---------------------------------------------------------*/


/* External variables --------------------------------------------------------*/


/* Functions -----------------------------------------------------------------*/
void AfeInit(void)
{

  


	// reset the spi stuff to spi2
	//spi_cs.gpio_pin = 25;
  spi_dev = device_get_binding("SPI_2");
	if (spi_dev == NULL) 
	{
		printk("Could not get %s device\n", "SPI_2");
		return;
	}
  else
  {
    printk("AFE SPI binding good\n");
  }

    if (spi_dev != NULL){
      printk("\nSPIM Device: %s\n", DT_PROP(MY_SPIM,label));
      printk("SPIM CSN PIN %d, MISO PIN d, MOSI PIN d, CLK PIN d\n",
                    DT_GPIO_PIN(MY_SPIM, cs_gpios)//,
                    // DT_PROP(MY_SPIM, miso_pin),
                    // DT_PROP(MY_SPIM, mosi_pin),
                    // DT_PROP(MY_SPIM, sck_pin)
                    ); 
    } //(spim_dev != NULL)
    else 
      printk("\nSPIM Device: %s\n", DT_PROP(MY_SPIM,label));


  // struct spi_cs_control *ctrl = SPI_CS_CONTROL_PTR_DT(DT_NODELABEL("SPI_2"), 25);
// struct spi_cs_control cs_ctrl = (struct spi_cs_control){
// 		.gpio_dev = device_get_binding(DT_GPIO_LABEL("SPI_2", 25)),
// 		.gpio_pin = DT_GPIO_PIN("SPI_2", 25),
// 		.gpio_dt_flags = DT_GPIO_FLAGS("SPI_2", 25),
// 		.delay = 0u,
// 	};

// static const struct gpio_dt_spec led = GPIO_DT_SPEC_GET(LED0_NODE, gpios);
// gpio_pin_configure_dt(&led, GPIO_OUTPUT_ACTIVE);
// gpio_pin_toggle_dt(&led);
// gpio_pin_toggle_dt(&led);
// gpio_pin_toggle_dt(&led);

  gpioPort0 = device_get_binding("GPIO_0");
////gpio_pin_configure(gpioPort0, 25, GPIO_OUTPUT_HIGH | GPIO_ACTIVE_LOW);
////gpio_pin_set(gpioPort0, 25, 1);

  gpioPort0 = device_get_binding("GPIO_0");
  spi_cs.gpio_dev = gpioPort0;

	afe_print_all();

	ecg_afe_init();
  ////gpio_pin_set(gpioPort0, 25, 0);

// float x = 4534.54354;
// printk("%2.4f \n",x);
// printf("Floating number: %3.2f\n", x);
}




void ecg_afe_init(void)
{
    static const unsigned char ecg_init_vals[NUM_AFE_REGISTERS_TO_INIT][2] = 
    {
        //=========== "Connect" button is pressed on GUI ==========
        {0x11, 0x11},       //Shutdown, Reset
        {DELAY, 50},
        {0x11, 0x02},       //Shutdown, Reset removed

        {0x20, 0x00},       //PPG MEAS disabled
        {0x21, 0x0C},       //PPG down: PPG2_PWRDN = 1, PPG1_PWRDN = 1
        {0xA0, 0x04},       //BIOZ confg: ECG_BIOZ_BG=1
        {0xAA, 0x00},       //BIOZ disabled

        {0x18, 0x00},       //PLL_EN=0
//        {0x19, 0xFF},       //MDIV = 255
        {0x1A, 0x00},       //BIOZ_NDIV = 0, BIOZ_KDIV = 0
//        {0x1B, 0x03},       //ECG_FDIV = 4
//        {0x1C, 0x40},       //ECG_NDIV = 64
        {0x1D, 0x20},       //CLK_FINE_TUNE
        {DELAY, 50},
        //-------- Total in this sections - 11 -------------

        //========== Custom Settings in GUI ================
        // PLL config
        {0x19, 0x7F},       //MDIV = 127
        {0x1B, 0x02},       //ECG_FDIV = 2
        {0x1C, 0x40},       //ECG_NDIV = 64
        {0x1D, 0x62},       //REF_CLK_SEL = External, CLK_FREQ_SEL = 32.768 kHz
        {0x18, 0x01},       //PLL_EN = 1
        {DELAY, 50},

 
        // ECG config
        {0x80, 0x06},       //ECG_DEC_RATE = 128 (SR = 256)
        {0x81, 0x02},       //ECG_PGA_GAIN = 1, ECG_INA_GAIN = 40
        {0x82, 0x01},       //ECG_MUX_SEL = 0x1
        {0x83, 0x3F},       //ECG_FAST_REC_THRESHOLD = 0x3D (61*2048 LSB, 95%)
        {0x90, 0x04},       //ECG_RBIAS_VALUE = 100 MOhm 
        {0x86, 0x00},       //ECG Calibration = all cleared
        {0x80, 0x07},       //ECG_EN = 1

        // AC lead off config
        {0x88, 0x49},       //EN_ECG_LOFF = 1, ECG_LOFF_MODE = 1, ECG_LOFF_FREQ = 2048
        {0x89, 0x60},       //ECG_LOFF_IMAG = 200 nA

        // RLD, Config
        {0x92, 0x4F},       //RLD_MODE = 1 (AC common mode feedback), ACTV_CM_P = 1, ACTV_CM_N = 1 (pos and neg input buffer enabled), RLD_GAIN = 100 V/V 
        {0x93, 0x43},       //RLD_SEL_ECG = 1 (input from ECGP and ECGN), BODY_BIAS_DAC = -120 mV
        {0x92, 0xCF},       //RLD_EN = 1

        // -------- Total in this section - 19 ----------------------

        //========= "Start" pressed in GUI ====================
        //{0xC0, 0x80},       //Interrupt 1 enabled A_FULL_EN = 1
        //{0x13, 0x01},       //Pin Functional Config

        {0x0D, 0xFA},       //FIFO configuration 1
        
        {0x20, 0x00},       //PPG MEAS disabled
        {0x21, 0x0C},       //PPG down: PPG2_PWRDN = 1, PPG1_PWRDN = 1
        {0xA0, 0x04},       //BIOZ confg: ECG_BIOZ_BG=1
        {0xAA, 0x00},       //BIOZ disabled
        {0x0E, 0x18},       //FIFO

        {0x11, 0x00},       //Shutdown removed!!
        {DELAY, 50}

        //-------- Total in this section - 8 -----------------------

        //-------- Total in three sections - 38 ---------------------
             
    };

    int err;
    static uint8_t tx_buffer[3];
    static uint8_t rx_buffer[3];

    printk("\033[m -- Configuration Init --\n");

    const struct spi_buf tx_buf = {
        .buf = tx_buffer,
        .len = sizeof(tx_buffer)
    };
    const struct spi_buf_set tx = {
        .buffers = &tx_buf,
        .count = 1
    };

    struct spi_buf rx_buf = {
        .buf = rx_buffer,
        .len = sizeof(rx_buffer),
    };
    const struct spi_buf_set rx = {
        .buffers = &rx_buf,
        .count = 1
    };

    tx_buffer[0] = 0x00;
    tx_buffer[1] = 0x80; // write
    rx_buffer[2] = 0x00; 

    for (int i = 0; i < NUM_AFE_REGISTERS_TO_INIT; i++)
    {

        if (ecg_init_vals[i][0] == DELAY)
        {
            printk("delay %dms\n", ecg_init_vals[i][1]);
            k_msleep(ecg_init_vals[i][1]);
        }
        else
        {
            tx_buffer[0] = ecg_init_vals[i][0];
            tx_buffer[1] = 0x00; // write
            tx_buffer[2] = ecg_init_vals[i][1]; 
            rx_buffer[2] = 0x00;

    //gpio_pin_set(gpioPort0, 25, 1);
		err = spi_transceive(spi_dev, &spi_cfg, &tx, &rx);
    //gpio_pin_set(gpioPort0, 25, 0);

            if (err) {
                printk("SPI error: %d\n", i);
            } else {
                printk("#%d, addr:%x value:%x\n", i, tx_buffer[0], tx_buffer[2]);
            }   

            tx_buffer[1] = 0x80; // read
            tx_buffer[2] = 0x00; // clear what we just sent
            rx_buffer[2] = 0x00;

    //gpio_pin_set(gpioPort0, 25, 1);
		err = spi_transceive(spi_dev, &spi_cfg, &tx, &rx);
    //gpio_pin_set(gpioPort0, 25, 0);

            if (err) {
                printk("SPI error: %d\n", i);
            } else {
                if (rx_buffer[2] != ecg_init_vals[i][1])
                {
                    printk("FAIL! #%d, addr:%x value:%x\n", i, tx_buffer[0], rx_buffer[2]);
                }
                else
                {
                    printk("SUCCESS #%d, addr:%x value:%x\n", i, tx_buffer[0], rx_buffer[2]);
                }
            }   
        }
    }

    printk("Finished the init of the registers\n\n");


	// check for a fifo flush
	// tx_buffer[0] = 0x0E;
	// tx_buffer[1] = 0x80; // read
	// tx_buffer[2] = 0x00; // clear what we just sent
	// rx_buffer[2] = 0x00;
	// err = spi_transceive(spi_dev, &spi_cfg, &tx, &rx);

	// __NOP();

	// tx_buffer[0] = 0x0E;
	// tx_buffer[1] = 0x00; // write
	// tx_buffer[2] = rx_buffer[2] | 0x10;
	// err = spi_transceive(spi_dev, &spi_cfg, &tx, &rx);

	// __NOP();

	//printk("Cleared FIFO buffer value:%x\n", tx_buffer[2]);

	//__NOP();
}


void afe_print_all(void)
{
	int err;
	static uint8_t tx_buffer[3];
	static uint8_t rx_buffer[3];
	//int i = 0;

	printk("\n\n-- Read all registers --\n");

	const struct spi_buf tx_buf = {
		.buf = tx_buffer,
		.len = sizeof(tx_buffer)
	};
	const struct spi_buf_set tx = {
		.buffers = &tx_buf,
		.count = 1
	};

	struct spi_buf rx_buf = {
		.buf = rx_buffer,
		.len = sizeof(rx_buffer),
	};
	const struct spi_buf_set rx = {
		.buffers = &rx_buf,
		.count = 1
	};



	for (int i = 0x00; i <= 0x04; i++)
	{
		tx_buffer[0] = i;
		tx_buffer[1] = 0x80; // read
		tx_buffer[2] = 0x00; // clear what we just sent
		rx_buffer[2] = 0x00;

    //gpio_pin_set(gpioPort0, 25, 1);
		err = spi_transceive(spi_dev, &spi_cfg, &tx, &rx);
    //gpio_pin_set(gpioPort0, 25, 0);
		if (err) {
			printk("SPI error: %d\n", i);
		} 
		else 
		{
			printk("Address: 0x%02X = 0x%02X\n", i, rx_buffer[2]);
		}	

	}


	for (int i = 0x08; i <= 0x0B; i++)
	{
		tx_buffer[0] = i;
		tx_buffer[1] = 0x80; // read
		tx_buffer[2] = 0x00; // clear what we just sent
		rx_buffer[2] = 0x00;

    //gpio_pin_set(gpioPort0, 25, 1);
		err = spi_transceive(spi_dev, &spi_cfg, &tx, &rx);
    //gpio_pin_set(gpioPort0, 25, 0);
		if (err) {
			printk("SPI error: %d\n", i);
		} 
		else 
		{
			printk("Address: 0x%02X = 0x%02X\n", i, rx_buffer[2]);
		}	

	}

	for (int i = 0x0D; i <= 0x0E; i++)
	{
		tx_buffer[0] = i;
		tx_buffer[1] = 0x80; // read
		tx_buffer[2] = 0x00; // clear what we just sent
		rx_buffer[2] = 0x00;

    //gpio_pin_set(gpioPort0, 25, 1);
		err = spi_transceive(spi_dev, &spi_cfg, &tx, &rx);
    //gpio_pin_set(gpioPort0, 25, 0);
		if (err) {
			printk("SPI error: %d\n", i);
		} 
		else 
		{
			printk("Address: 0x%02X = 0x%02X\n", i, rx_buffer[2]);
		}	

	}


	for (int i = 0x10; i <= 0x1D; i++)
	{
		tx_buffer[0] = i;
		tx_buffer[1] = 0x80; // read
		tx_buffer[2] = 0x00; // clear what we just sent
		rx_buffer[2] = 0x00;

    //gpio_pin_set(gpioPort0, 25, 1);
		err = spi_transceive(spi_dev, &spi_cfg, &tx, &rx);
    //gpio_pin_set(gpioPort0, 25, 0);
		if (err) {
			printk("SPI error: %d\n", i);
		} 
		else 
		{
			printk("Address: 0x%02X = 0x%02X\n", i, rx_buffer[2]);
		}	

	}

	for (int i = 0x20; i <= 0x24; i++)
	{
		tx_buffer[0] = i;
		tx_buffer[1] = 0x80; // read
		tx_buffer[2] = 0x00; // clear what we just sent
		rx_buffer[2] = 0x00;

    //gpio_pin_set(gpioPort0, 25, 1);
		err = spi_transceive(spi_dev, &spi_cfg, &tx, &rx);
    //gpio_pin_set(gpioPort0, 25, 0);
		if (err) {
			printk("SPI error: %d\n", i);
		} 
		else 
		{
			printk("Address: 0x%02X = 0x%02X\n", i, rx_buffer[2]);
		}	

	}

	for (int i = 0x28; i <= 0x29; i++)
	{
		tx_buffer[0] = i;
		tx_buffer[1] = 0x80; // read
		tx_buffer[2] = 0x00; // clear what we just sent
		rx_buffer[2] = 0x00;

    //gpio_pin_set(gpioPort0, 25, 1);
		err = spi_transceive(spi_dev, &spi_cfg, &tx, &rx);
    //gpio_pin_set(gpioPort0, 25, 0);
		if (err) {
			printk("SPI error: %d\n", i);
		} 
		else 
		{
			printk("Address: 0x%02X = 0x%02X\n", i, rx_buffer[2]);
		}	

	}

	for (int i = 0x30; i <= 0x5F; i++)
	{
		tx_buffer[0] = i;
		tx_buffer[1] = 0x80; // read
		tx_buffer[2] = 0x00; // clear what we just sent
		rx_buffer[2] = 0x00;

    //gpio_pin_set(gpioPort0, 25, 1);
		err = spi_transceive(spi_dev, &spi_cfg, &tx, &rx);
    //gpio_pin_set(gpioPort0, 25, 0);
		if (err) {
			printk("SPI error: %d\n", i);
		} 
		else 
		{
			printk("Address: 0x%02X = 0x%02X\n", i, rx_buffer[2]);
		}	

	}

	for (int i = 0x70; i <= 0x75; i++)
	{
		tx_buffer[0] = i;
		tx_buffer[1] = 0x80; // read
		tx_buffer[2] = 0x00; // clear what we just sent
		rx_buffer[2] = 0x00;

    //gpio_pin_set(gpioPort0, 25, 1);
		err = spi_transceive(spi_dev, &spi_cfg, &tx, &rx);
    //gpio_pin_set(gpioPort0, 25, 0);
		if (err) {
			printk("SPI error: %d\n", i);
		} 
		else 
		{
			printk("Address: 0x%02X = 0x%02X\n", i, rx_buffer[2]);
		}	

	}

	for (int i = 0x80; i <= 0x89; i++)
	{
		tx_buffer[0] = i;
		tx_buffer[1] = 0x80; // read
		tx_buffer[2] = 0x00; // clear what we just sent
		rx_buffer[2] = 0x00;

    //gpio_pin_set(gpioPort0, 25, 1);
		err = spi_transceive(spi_dev, &spi_cfg, &tx, &rx);
    //gpio_pin_set(gpioPort0, 25, 0);
		if (err) {
			printk("SPI error: %d\n", i);
		} 
		else 
		{
			printk("Address: 0x%02X = 0x%02X\n", i, rx_buffer[2]);
		}	

	}

	for (int i = 0x90; i <= 0x93; i++)
	{
		tx_buffer[0] = i;
		tx_buffer[1] = 0x80; // read
		tx_buffer[2] = 0x00; // clear what we just sent
		rx_buffer[2] = 0x00;

    //gpio_pin_set(gpioPort0, 25, 1);
		err = spi_transceive(spi_dev, &spi_cfg, &tx, &rx);
    //gpio_pin_set(gpioPort0, 25, 0);
		if (err) {
			printk("SPI error: %d\n", i);
		} 
		else 
		{
			printk("Address: 0x%02X = 0x%02X\n", i, rx_buffer[2]);
		}	

	}

	for (int i = 0xA0; i <= 0xA9; i++)
	{
		tx_buffer[0] = i;
		tx_buffer[1] = 0x80; // read
		tx_buffer[2] = 0x00; // clear what we just sent
		rx_buffer[2] = 0x00;

    //gpio_pin_set(gpioPort0, 25, 1);
		err = spi_transceive(spi_dev, &spi_cfg, &tx, &rx);
    //gpio_pin_set(gpioPort0, 25, 0);
		if (err) {
			printk("SPI error: %d\n", i);
		} 
		else 
		{
			printk("Address: 0x%02X = 0x%02X\n", i, rx_buffer[2]);
		}	

	}


	for (int i = 0xAA; i <= 0xAD; i++)
	{
		tx_buffer[0] = i;
		tx_buffer[1] = 0x80; // read
		tx_buffer[2] = 0x00; // clear what we just sent
		rx_buffer[2] = 0x00;

    //gpio_pin_set(gpioPort0, 25, 1);
		err = spi_transceive(spi_dev, &spi_cfg, &tx, &rx);
    //gpio_pin_set(gpioPort0, 25, 0);
		if (err) {
			printk("SPI error: %d\n", i);
		} 
		else 
		{
			printk("Address: 0x%02X = 0x%02X\n", i, rx_buffer[2]);
		}	

	}


	for (int i = 0xB0; i <= 0xB1; i++)
	{
		tx_buffer[0] = i;
		tx_buffer[1] = 0x80; // read
		tx_buffer[2] = 0x00; // clear what we just sent
		rx_buffer[2] = 0x00;

    //gpio_pin_set(gpioPort0, 25, 1);
		err = spi_transceive(spi_dev, &spi_cfg, &tx, &rx);
    //gpio_pin_set(gpioPort0, 25, 0);
		if (err) {
			printk("SPI error: %d\n", i);
		} 
		else 
		{
			printk("Address: 0x%02X = 0x%02X\n", i, rx_buffer[2]);
		}	

	}

	for (int i = 0xB4; i <= 0xB4; i++)
	{
		tx_buffer[0] = i;
		tx_buffer[1] = 0x80; // read
		tx_buffer[2] = 0x00; // clear what we just sent
		rx_buffer[2] = 0x00;

    //gpio_pin_set(gpioPort0, 25, 1);
		err = spi_transceive(spi_dev, &spi_cfg, &tx, &rx);
    //gpio_pin_set(gpioPort0, 25, 0);
		if (err) {
			printk("SPI error: %d\n", i);
		} 
		else 
		{
			printk("Address: 0x%02X = 0x%02X\n", i, rx_buffer[2]);
		}	

	}	

	for (int i = 0xB6; i <= 0xB6; i++)
	{
		tx_buffer[0] = i;
		tx_buffer[1] = 0x80; // read
		tx_buffer[2] = 0x00; // clear what we just sent
		rx_buffer[2] = 0x00;

    //gpio_pin_set(gpioPort0, 25, 1);
		err = spi_transceive(spi_dev, &spi_cfg, &tx, &rx);
    //gpio_pin_set(gpioPort0, 25, 0);
		if (err) {
			printk("SPI error: %d\n", i);
		} 
		else 
		{
			printk("Address: 0x%02X = 0x%02X\n", i, rx_buffer[2]);
		}	

	}

	for (int i = 0xC0; i <= 0xC9; i++)
	{
		tx_buffer[0] = i;
		tx_buffer[1] = 0x80; // read
		tx_buffer[2] = 0x00; // clear what we just sent
		rx_buffer[2] = 0x00;

    //gpio_pin_set(gpioPort0, 25, 1);
		err = spi_transceive(spi_dev, &spi_cfg, &tx, &rx);
    //gpio_pin_set(gpioPort0, 25, 0);
		if (err) {
			printk("SPI error: %d\n", i);
		} 
		else 
		{
			printk("Address: 0x%02X = 0x%02X\n", i, rx_buffer[2]);
		}	

	}



	for (int i = 0xFF; i <= 0xFF; i++)
	{
		tx_buffer[0] = i;
		tx_buffer[1] = 0x80; // read
		tx_buffer[2] = 0x00; // clear what we just sent
		rx_buffer[2] = 0x00;

    //gpio_pin_set(gpioPort0, 25, 1);
		err = spi_transceive(spi_dev, &spi_cfg, &tx, &rx);
    //gpio_pin_set(gpioPort0, 25, 0);
		if (err) {
			printk("SPI error: %d\n", i);
		} 
		else 
		{
			printk("Address: 0x%02X = 0x%02X\n", i, rx_buffer[2]);
		}	

	}





	__NOP();

	printk("\nFinished reading all registes\n\n\n");

	__NOP();
}


uint32_t ecg_read(int16_t array[])
{
	uint32_t counter = 0;


	int err;
	static uint8_t tx_buffer[1000];
	static uint8_t rx_buffer[1000];
	static uint8_t header = 0;
	static uint32_t data = 0;

	//static uint32_t i = 0;

	//printk("-- Read stuff --\n");

	struct spi_buf tx_buf = {
		.buf = tx_buffer,
		.len = sizeof(tx_buffer)
	};
	const struct spi_buf_set tx = {
		.buffers = &tx_buf,
		.count = 1
	};

	struct spi_buf rx_buf = {
		.buf = rx_buffer,
		.len = sizeof(rx_buffer),
	};
	const struct spi_buf_set rx = {
		.buffers = &rx_buf,
		.count = 1
	};

	tx_buffer[0] = 0x0A;
	tx_buffer[1] = 0x80;
	data = 0;

	tx_buf.len = 4;
	rx_buf.len = 4;

    //gpio_pin_set(gpioPort0, 25, 1);
		err = spi_transceive(spi_dev, &spi_cfg, &tx, &rx);
    //gpio_pin_set(gpioPort0, 25, 0);
	if (err) {
		//printk("SPI error: %d\n", err);
	} 
	else
	{
		counter = rx_buffer[3];
		if ((rx_buffer[2] & 0x80) != 0)
		{
			counter = counter + 0x0100;
		}
	}

	if (counter > 0)
	{

		tx_buf.len = (counter * 3) + 2;
		rx_buf.len = (counter * 3) + 2;

		tx_buffer[0] = 0x0C;
		tx_buffer[1] = 0x80;
		data = 0;

    //gpio_pin_set(gpioPort0, 25, 1);
		err = spi_transceive(spi_dev, &spi_cfg, &tx, &rx);
    //gpio_pin_set(gpioPort0, 25, 0);
		if (err) {
			//printk("SPI error: %d\n", err);
		} 
		else
		{
			int index = 2;
			for (int x = 0; x < counter; x++)
			{
				header = (rx_buffer[index] & 0xFC);
				if ((header & 0xf0) != 0xB0)
				{
					//printk("Header not B error: %X\n", header);
				}
				else
				{
					data = (rx_buffer[index] & 0x03);
					data = data << 8;
					data = data + rx_buffer[index+1];
					data = data << 8;
					data = data + rx_buffer[index+2];
					//__NOP();
					array[x] = (int)(data >> 2);
					index = index + 3;
				}
			}
		}
	
	}


	// else 
	// {
	// 	header = (rx_buffer[2] & 0xFC);

	// 	if ((header & 0xf0) != 0xB0)
	// 	{
	// 		printk("Header not B error: %X\n", header);
	// 	}
	// 	else
	// 	{
	// 		data = (rx_buffer[2] & 0x03);
	// 		data = data << 8;
	// 		data = data + rx_buffer[3];
	// 		data = data << 8;
	// 		data = data + rx_buffer[4];
	// 		__NOP();
	// 	}
	// }
	//total = counter;
	return counter;

}



/*****END OF FILE****/
